"""
Autogenerated using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
hub.exec.boto3.client.ec2.create_flow_logs
hub.exec.boto3.client.ec2.delete_flow_logs
hub.exec.boto3.client.ec2.describe_flow_logs
"""
import copy
from typing import Any
from typing import Dict
from typing import List

__contracts__ = ["resource"]

TREQ = {
    "present": {
        "require": ["aws.ec2.vpc.present"],
        "require": ["aws.ec2.subnet.present"],
    },
}


async def present(
    hub,
    ctx,
    name: str,
    resource_type: str,
    traffic_type: str,
    log_format: str,
    max_aggregation_interval: int,
    resource_id: str = None,
    log_destination: str = None,
    log_destination_type: str = None,
    log_group_name: str = None,
    iam_role: str = None,
    destination_options: Dict = None,
    resource_ids: List = None,
    tags: Dict[str, str] = None,
) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    Creates a flow log to capture information about IP traffic for a specific network interface, subnet,
    or VPC.  Flow log data for a monitored network interface is recorded as flow log records, which are log events
    consisting of fields that describe the traffic flow. For more information, see Flow log records in the Amazon
    Virtual Private Cloud User Guide. When publishing to CloudWatch Logs, flow log records are published to a log
    group, and each network interface has a unique log stream in the log group. When publishing to Amazon S3, flow
    log records for all of the monitored network interfaces are published to a single log file object that is stored
    in the specified bucket. For more information, see VPC Flow Logs in the Amazon Virtual Private Cloud User Guide.
    Args:
        hub:
        ctx:
        name(str): Name of the resource.
        log_group_name(str, Optional): log_group_name if the log_destination_type is cloudwatch
        resource_type(str): Type of resource flow-log is attached to (Subnet, VPC, NetworkInterface)
        traffic_type(str): Type of traffic to be recorded (REJECT, ALL, ACCEPT)
        log_destination_type(str, Optional): S3 bucket or Default: cloud-watch-logs
        log_destination(str, Optional): S3 bucket ARN
        log_format(str): Syntax to be used to print log statements
        max_aggregation_interval(int): Max interval during which packets are aggregated and then stored in log record
        resource_id(str, Optional): AWS Flow log ID to identify the resource
        iam_role(str, Optional): ARN of IAM role to be used to post in cloud-watch-logs
        destination_options(Dict, Optional): The destination options.
                a) FileFormat(str) : The format for the flow log. The default is plain-text.
                b) HiveCompatiblePartitions (bool) : Indicates whether to use Hive-compatible prefixes for flow logs
                    stored in Amazon S3. The default is false .
                c) PerHourPartition (bool) : Indicates whether to partition the flow log per hour. This reduces the cost
                    and response time for queries. The default is false .
        resource_ids([str]): list of resource_ids flow-log is attached to
        tags(Dict, Optional): List of Name, Value pair to tag flow-logs

    Request Syntax:
        [flow-log-id]:
          aws.ec2.flow_log.present:
          - name: 'string'
          - log_group_name: 'string'
          - resource_type: 'integer'
          - traffic_type: 'string'
          - tags:
            - Key: 'string'
              Value: 'string'

    Returns:
        Dict[str, Any]

    Examples:
        .. code-block:: sls
            fl-09c0787e693332a0a:
                aws.ec2.flow_log.present:
                - traffic_type: REJECT
                - log_destination_type: s3
    """

    result = dict(comment="", old_state=None, new_state=None, name=name, result=True)
    before = None
    resource_updated = False
    if resource_id:
        before = await hub.exec.boto3.client.ec2.describe_flow_logs(
            ctx, FlowLogIds=[resource_id]
        )

    if before and before["result"] and before["ret"].get("FlowLogs"):
        result[
            "old_state"
        ] = hub.tool.aws.ec2.conversion_utils.convert_raw_flow_log_to_present(
            before["ret"]["FlowLogs"][0]
        )
        plan_state = copy.deepcopy(result["old_state"])
        old_tags = before["ret"].get("FlowLogs")[0].get("Tags")
        if tags is not None:
            if not hub.tool.aws.state_comparison_utils.are_lists_identical(
                tags, old_tags
            ):
                result["comment"] = f"'{name}' already exists. Updating tags"
                update_ret = await hub.exec.aws.ec2.tag.update_tags(
                    ctx,
                    resource_id=resource_id,
                    old_tags=old_tags,
                    new_tags=tags,
                )
                if not update_ret["result"]:
                    result["comment"] = update_ret["comment"]
                    result["result"] = False
                if ctx.get("test", False) and update_ret["ret"] is not None:
                    plan_state["tags"] = update_ret["ret"].get("tags")
                    result["comment"] = f"Would update aws.ec2.flow_log {name}"
                resource_updated = resource_updated or bool(update_ret["result"])
    else:
        if ctx.get("test", False):
            result["new_state"] = hub.tool.aws.test_state_utils.generate_test_state(
                enforced_state={},
                desired_state={
                    "name": name,
                    "resource_type": resource_type,
                    "traffic_type": traffic_type,
                    "log_format": log_format,
                    "max_aggregation_interval": max_aggregation_interval,
                    "resource_id": resource_id,
                    "log_destination": log_destination,
                    "log_destination_type": log_destination_type,
                    "log_group_name": log_group_name,
                    "iam_role": iam_role,
                    "destination_options": destination_options,
                    "resource_ids": resource_ids,
                    "tags": tags,
                },
            )
            result["comment"] = f"Would create aws.ec2.flow_log {name}"
            return result
        try:
            ret = await hub.exec.boto3.client.ec2.create_flow_logs(
                ctx,
                DeliverLogsPermissionArn=iam_role,
                TagSpecifications=[{"ResourceType": "vpc-flow-log", "Tags": tags}]
                if tags is not None
                else None,
                DestinationOptions=destination_options,
                LogDestination=log_destination,
                LogDestinationType=log_destination_type,
                LogFormat=log_format,
                LogGroupName=log_group_name,
                MaxAggregationInterval=max_aggregation_interval,
                ResourceIds=resource_ids,
                ResourceType=resource_type,
                TrafficType=traffic_type,
            )
            result["result"] = ret["result"]
            if not result["result"]:
                result["comment"] = ret["comment"]
                return result
            result["comment"] = f"Created '{name}'"
            resource_id = ret["ret"]["FlowLogIds"][0]
            resource_updated = True
        except hub.tool.boto3.exception.ClientError as e:
            result["comment"] = f"{e.__class__.__name__}: {e}"

    if ctx.get("test", False):
        result["new_state"] = plan_state
    elif resource_updated:
        after = await hub.exec.boto3.client.ec2.describe_flow_logs(
            ctx, FlowLogIds=[resource_id]
        )
        if after and after["result"] and after["ret"].get("FlowLogs"):
            result[
                "new_state"
            ] = hub.tool.aws.ec2.conversion_utils.convert_raw_flow_log_to_present(
                after["ret"]["FlowLogs"][0]
            )
    else:
        result["new_state"] = copy.deepcopy(result["old_state"])
    return result


async def absent(hub, ctx, name: str, resource_id: str) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    Deletes a flow log.
    Args:
        hub:
        ctx:
        name(Text): Name of the resource.
        resource_id(str): AWS Flow log ID to identify the resource

    Returns:
        Dict[str, Any]
    Examples:
        .. code-block:: sls
            resource_is_absent:
              aws_auto.ec2.flow_log.absent:
                - name: value
                - resource_id: value
    """

    result = dict(comment="", old_state=None, new_state=None, name=name, result=True)
    before = await hub.exec.boto3.client.ec2.describe_flow_logs(
        ctx, FlowLogIds=[resource_id]
    )

    if not before:
        result["comment"] = f"'{name}' already absent"
    elif ctx.get("test", False):
        result["comment"] = f"Would delete aws.ec2.flow_log {name}"
    else:
        try:
            ret = await hub.exec.boto3.client.ec2.delete_flow_logs(
                ctx, FlowLogIds=[resource_id]
            )
            result["result"] = ret["result"]
            if not result["result"]:
                result["comment"] = f"'{name}' is already in deleted state."
                return result
            result["comment"] = f"Deleted '{name}'"
        except hub.tool.boto3.exception.ClientError as e:
            result["comment"] = f"{e.__class__.__name__}: {e}"
    if before and before["result"] and before["ret"].get("FlowLogs"):
        result[
            "old_state"
        ] = hub.tool.aws.ec2.conversion_utils.convert_raw_flow_log_to_present(
            before["ret"]["FlowLogs"][0]
        )
    return result


async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
    r"""
    **Autogenerated function**
    Describe the resource in a way that can be recreated/managed with the corresponding "present" function

    Describes one or more flow logs. To view the information in your flow logs (the log streams for the network
    interfaces), you must use the CloudWatch Logs console or the CloudWatch Logs API.
    Returns:
        Dict[str, Any]
    Examples:
        .. code-block:: bash
            $ idem describe aws_auto.ec2.flow_log
    """

    result = {}
    ret = await hub.exec.boto3.client.ec2.describe_flow_logs(ctx)
    if not ret["result"]:
        hub.log.debug(f"Could not describe flow_log {ret['comment']}")
        return result
    for flow_log in ret["ret"]["FlowLogs"]:
        translated_resource = (
            hub.tool.aws.ec2.conversion_utils.convert_raw_flow_log_to_present(flow_log)
        )

        result[flow_log["FlowLogId"]] = {
            "aws.ec2.flow_log.present": [
                {parameter_key: parameter_value}
                for parameter_key, parameter_value in translated_resource.items()
            ]
        }
    return result



"""
Autogenerated state module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__

"""
import copy
from typing import Any
from typing import Dict

__contracts__ = ["resource"]

TREQ = {
    "absent": {
        "require": [
            "aws.s3.bucket.absent",
        ],
    },
    "present": {
        "require": [
            "aws.s3.bucket.present",
        ],
    },
}


async def present(
    hub,
    ctx,
    name: str,
    bucket: str,
    policy: str,
    confirm_remove_self_bucket_access: bool = None,
    expected_bucket_owner: str = None,
) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    Applies an Amazon S3 bucket policy to an Amazon S3 bucket. If you are using an identity other than the root user of
    the Amazon Web Services account that owns the bucket, the calling identity must have the PutBucketPolicy permissions
     on the specified bucket and belong to the bucket owner's account in order to use this operation.

    If you don't have PutBucketPolicy permissions, Amazon S3 returns a 403 Access Denied error.
    If you have the correct permissions, but you're not using an identity that belongs to the bucket owner's account,
    Amazon S3 returns a 405 Method Not Allowed error.

    Args:
        hub:
        ctx:
        name(Text): The name of the bucket policy.
        bucket(string):  The name of the bucket
        policy (string): The bucket policy as a JSON document.
        confirm_remove_self_bucket_access (boolean, optional): Set this parameter to true to confirm that you want to remove your permissions to change this bucket policy in the future.
        expected_bucket_owner (string, optional): The account ID of the expected bucket owner. If the bucket is owned by a different account, the request will fail with an HTTP 403 (Access Denied) error.

    Request Syntax:
        resource_name:
          aws.s3.bucket_policy.present:
            - bucket: string
            - policy: string
            - confirm_remove_self_bucket_access: boolean
            - expected_bucket_owner: string

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            test_bucket-bb7bb32e9533-policy:
              aws.s3.bucket_policy.present:
                - bucket: test_bucket-bb7bb32e9533
                - policy: '{"Version":"2012-10-17","Statement":[{"Sid":"PublicReadGetObject","Effect":"Allow","Principal":{"AWS":"arn:aws:iam::418235808912:root"},"Action":"s3:GetObject","Resource":"arn:aws:s3:::thebugbucket3/*"}]}'
                - confirm_remove_self_bucket_access: False
                - expected_bucket_owner: 1239234249


    """
    result = dict(comment="", old_state=None, new_state=None, name=name, result=True)
    resource = hub.tool.boto3.resource.create(ctx, "s3", "Bucket", bucket)
    before = await hub.tool.boto3.resource.describe(resource)
    if not before:
        result["comment"] = f"s3 bucket '{bucket}' does not exist."
        result["result"] = False
    else:
        # get policy of the bucket if exists
        bucket_policy = hub.tool.boto3.resource.create(
            ctx, "s3", "BucketPolicy", bucket
        )
        policy_ret = await hub.tool.boto3.resource.describe(bucket_policy)
        old_policy = policy_ret.get("Policy") if policy_ret else None
        before["policy"] = old_policy
        result["old_state"] = before
        if ctx.get("test", False):
            if old_policy != policy:
                result["comment"] = f"Would update aws.s3.bucket {bucket} policy"
            else:
                result["comment"] = f"No changes were made to policy"
            return result
        # create or update bucket policy only if there is any change in policy
        if old_policy != policy:
            try:
                # create or update the bucket policy
                update_ret = await hub.exec.boto3.client.s3.put_bucket_policy(
                    ctx,
                    Bucket=bucket,
                    Policy=policy,
                    ConfirmRemoveSelfBucketAccess=confirm_remove_self_bucket_access,
                    ExpectedBucketOwner=expected_bucket_owner,
                )
                if not update_ret["result"]:
                    result["comment"] = update_ret["comment"]
                    result["result"] = False
                    return result
            except Exception as e:
                result["comment"] = str(e)
                result["result"] = False
                return result
            try:
                new_policy_ret = await hub.tool.boto3.resource.describe(bucket_policy)
                new_policy = new_policy_ret.get("Policy") if new_policy_ret else None
                after = copy.deepcopy(before)
                after["policy"] = new_policy
                result["new_state"] = after
                result[
                    "comment"
                ] = f"Created bucket policy {bucket}-policy on bucket {bucket}"
            except Exception as e:
                result["comment"] = str(e)
                result["result"] = False
        else:
            result["new_state"] = copy.deepcopy(before)
            result["comment"] = f"No changes were made to policy"
    return result


async def absent(
    hub, ctx, name: str, bucket: str, expected_bucket_owner: str = None
) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    Deletes the policy of specified s3 bucket.

    Args:
        name(Text): The name of the s3 bucket policy.
        bucket(string): The name of the s3 bucket.
        expected_bucket_owner(string): The account ID of the expected bucket owner. If the bucket is owned by a different account, the request will fail with an HTTP 403 (Access Denied) error.

    Returns:
        Dict[str, Any]

    Request Syntax:
        resource_name:
          aws.s3.bucket_policy.absent:
            - name: string
            - bucket: string
            - expected_bucket_owner: string

    Examples:

        .. code-block:: sls

            bucket-5435423646-456464-policy:
              aws.s3.bucket_policy.absent:
                - name: bucket-5435423646-456464-policy
                - bucket: bucket-5435423646-456464
                - expected_bucket_owner: 1239234249
    """

    result = dict(comment="", old_state=None, new_state=None, name=name, result=True)
    resource = hub.tool.boto3.resource.create(ctx, "s3", "Bucket", bucket)
    before = await hub.tool.boto3.resource.describe(resource)

    if not before:
        result["comment"] = f"s3 bucket '{bucket}' does not exist."
        result["result"] = False
    else:
        try:
            resource = hub.tool.boto3.resource.create(ctx, "s3", "BucketPolicy", bucket)
            policy_ret = await hub.tool.boto3.resource.describe(resource)
            old_policy = (
                policy_ret["Policy"]
                if policy_ret and policy_ret.get("Policy")
                else None
            )
            before["policy"] = old_policy
            if ctx.get("test", False):
                if old_policy:
                    result["comment"] = f"Would delete aws.s3.bucket_policy {name}"
                else:
                    result["comment"] = f"Policy is already absent for bucket {bucket}"
                return result
            if old_policy:
                ret = await hub.exec.boto3.client.s3.delete_bucket_policy(
                    ctx, Bucket=bucket, ExpectedBucketOwner=expected_bucket_owner
                )
                result["result"] = ret["result"]
                if not result["result"]:
                    result["comment"] = ret["comment"]
                    result["result"] = False
                    return result
                result["comment"] = f"Deleted policy '{name}' for bucket {bucket}"
            else:
                result["comment"] = f"Policy is already absent for bucket {bucket}"
        except hub.tool.boto3.exception.ClientError as e:
            result["comment"] = f"{e.__class__.__name__}: {e}"
        after = copy.deepcopy(before)
        after["policy"] = None
        result["old_state"] = before
        result["new_state"] = after

    return result


async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
    """
    Obtain S3 bucket policy for each bucket under the given context for any user.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: bash

            $ idem describe aws.s3.bucket_policy
    """
    result = {}
    # To describe the policy of all the buckets, we first need to list all the buckets, then get the
    # policy of each bucket
    ret = await hub.exec.boto3.client.s3.list_buckets(ctx)

    if not ret["result"]:
        hub.log.debug(f"Could not describe S3 buckets {ret['comment']}")
        return {}

    for bucket in ret["ret"]["Buckets"]:
        resource_id = bucket.get("Name")
        resource_translated = [{"bucket": resource_id}]
        resource_name = f"{resource_id}-policy"
        # get policy for each bucket
        try:
            resource = hub.tool.boto3.resource.create(
                ctx, "s3", "BucketPolicy", resource_id
            )
            policy_ret = await hub.tool.boto3.resource.describe(resource)
            if policy_ret and policy_ret["Policy"]:
                resource_translated.append({"policy": policy_ret["Policy"]})
            result[resource_name] = {
                "aws.s3.bucket_policy.present": resource_translated
            }
        except Exception as e:
            hub.log.warning(
                f"Could not get attached policy for bucket {resource_id} with error"
                f" {e} . Describe will skip this bucket and continue."
            )

    return result



"""
Autogenerated using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__

hub.exec.boto3.client.eks.create_cluster
hub.exec.boto3.client.eks.delete_cluster
hub.exec.boto3.client.eks.deregister_cluster
hub.exec.boto3.client.eks.describe_cluster
hub.exec.boto3.client.eks.list_clusters
hub.exec.boto3.client.eks.register_cluster
"""
import copy
import time
from collections import OrderedDict
from typing import Any
from typing import Dict

from dict_tools import differ

__contracts__ = ["resource"]

TREQ = {
    "absent": {
        "require": [
            "aws.ec2.subnet.absent",
            "aws.iam.role.absent",
        ],
    },
    "present": {
        "require": [
            "aws.ec2.subnet.present",
            "aws.iam.role.present",
        ],
    },
}

create_attempts = 60
update_attempts = 120


async def present(
    hub,
    ctx,
    name: str,
    role_arn: str,
    version: str = None,
    resources_vpc_config: Dict = None,
    kubernetes_network_config: Dict = None,
    logging: Dict = None,
    encryption_config: Dict = None,
    tags: Dict[str, str] = None,
) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    Creates an Amazon EKS control plane.  The Amazon EKS control plane consists of control plane instances that run
    the Kubernetes software, such as etcd and the API server. The control plane runs in an account managed by Amazon
    Web Services, and the Kubernetes API is exposed by the Amazon EKS API server endpoint. Each Amazon EKS cluster
    control plane is single tenant and unique. It runs on its own set of Amazon EC2 instances. The cluster control
    plane is provisioned across multiple Availability Zones and fronted by an Elastic Load Balancing Network Load
    Balancer. Amazon EKS also provisions elastic network interfaces in your VPC subnets to provide connectivity from
    the control plane instances to the nodes (for example, to support kubectl exec, logs, and proxy data flows).
    Amazon EKS nodes run in your Amazon Web Services account and connect to your cluster's control plane over the
    Kubernetes API server endpoint and a certificate file that is created for your cluster. In most cases, it takes
    several minutes to create a cluster. After you create an Amazon EKS cluster, you must configure your Kubernetes
    tooling to communicate with the API server and launch nodes into your cluster. For more information, see
    Managing Cluster Authentication and Launching Amazon EKS nodes in the Amazon EKS User Guide.

    Args:
        name(Text): A name, ID to identify the resource.
        version(TEXT): The desired Kubernetes version for your cluster
        role_arn(TEXT): The Amazon Resource Name (ARN) of the IAM role that provides permissions for the Kubernetes
            control plane to make calls to Amazon Web Services API operations on your behalf. For more information,
            see Amazon EKS Service IAM Role in the * Amazon EKS User Guide *
        resources_vpc_config(Dict, required): The VPC configuration that's used by the cluster control plane. Amazon
            EKS VPC resources have specific requirements to work properly with Kubernetes. For more information,
            see Cluster VPC Considerations and Cluster Security Group Considerations in the Amazon EKS User Guide . You
            must specify at least two subnets. You can specify up to five security groups. However, we recommend that you
            use a dedicated security group for your cluster control plane.
        kubernetes_network_config(Dict, optional): The Kubernetes network configuration for the cluster.
        logging(Dict, optional): Enable or disable exporting the Kubernetes control plane logs for your cluster to
            CloudWatch Logs. By default, cluster control plane logs aren't exported to CloudWatch Logs. For more
            information, see Amazon EKS Cluster control plane logs in the * Amazon EKS User Guide *
        encryption_config(Dict, optional): The encryption configuration for the cluster.
        tags(Dict, optional): The metadata to apply to the cluster to assist with categorization and organization.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            my_cluster:
              aws.eks.cluster.present:
              - role_arn: arn:aws:iam::202236654445:role/ek1
              - version: '1.21'
              - resources_vpc_config:
                  endpointPrivateAccess: true
                  endpointPublicAccess: false
                  publicAccessCidrs: []
                  securityGroupIds:
                  - sg-000b8c21b6b480577
                  subnetIds:
                  - subnet-07bc4715744fb7b39
                  - subnet-0aa66edffb00d8881
              - kubernetes_network_config:
                  ipFamily: ipv4
                  serviceIpv4Cidr: 172.20.0.0/16
              - logging:
                  clusterLogging:
                  - enabled: true
                    types:
                    - api
                    - audit
                  - enabled: false
                    types:
                    - authenticator
                    - controllerManager
                    - scheduler
              - tags:
                  one: '1'
                  two: '2'

    """

    result = dict(comment="", old_state=None, new_state=None, name=name, result=True)
    is_changed = False
    before = await hub.exec.boto3.client.eks.describe_cluster(ctx, name=name)
    if ctx.get("test", False):
        if before:
            result["comment"] = f"Would update aws.eks.cluster {name}"
            result["result"] = True
        else:
            result["comment"] = f"Would create aws.eks.cluster {name}"
            result["result"] = True
        return result

    try:
        if before:
            result["comment"] = f"'{name}' already exists"
            result["old_state"] = before["ret"]["cluster"]
            arn = before["ret"]["cluster"]["arn"]
            if before["ret"]["cluster"].get("clientRequestToken"):
                client_request_token = before["ret"]["cluster"]["clientRequestToken"]
            else:
                client_request_token = None
            # change depends on resourcesVpcConfig,  encryptionConfig, logging and version
            if version and before["ret"]["cluster"].get("version") != version:
                ret = await hub.exec.boto3.client.eks.update_cluster_version(
                    ctx,
                    name=name,
                    version=version,
                    clientRequestToken=client_request_token,
                )
                result["result"] = ret["result"]
                if not result["result"]:
                    result["comment"] = ret["comment"]
                    return result
                time.sleep(60)
                await hub.tool.boto3.client.wait(
                    ctx,
                    "eks",
                    "cluster_active",
                    None,
                    name=name,
                    WaiterConfig={"MaxAttempts": update_attempts},
                )
                is_changed = True

            if (
                encryption_config
                and before["ret"]["cluster"].get("encryptionConfig")
                != encryption_config
            ):
                ret = await hub.exec.boto3.client.eks.associate_encryption_config(
                    ctx,
                    clusterName=name,
                    encryptionConfig=encryption_config,
                    clientRequestToken=client_request_token,
                )
                result["result"] = ret["result"]
                if not result["result"]:
                    result["comment"] = ret["comment"]
                    return result
                await hub.tool.boto3.client.wait(
                    ctx,
                    "eks",
                    "cluster_active",
                    None,
                    name=name,
                    WaiterConfig={"MaxAttempts": update_attempts},
                )
                is_changed = True

            if logging:
                if differ.deep_diff(before["ret"]["cluster"].get("logging"), logging):
                    ret = await hub.exec.boto3.client.eks.update_cluster_config(
                        ctx,
                        name=name,
                        logging=logging,
                        clientRequestToken=client_request_token,
                    )
                    result["result"] = ret["result"]
                    if not result["result"]:
                        result["comment"] = ret["comment"]
                        return result
                    await hub.tool.boto3.client.wait(
                        ctx,
                        "eks",
                        "cluster_active",
                        None,
                        name=name,
                        WaiterConfig={"MaxAttempts": update_attempts},
                    )
                    time.sleep(60)
                    is_changed = True
            vpc_config_changes = hub.tool.aws.eks_utils.get_resource_vpc_config_changes(
                resources_vpc_config,
                before["ret"]["cluster"].get("resourcesVpcConfig"),
            )
            if vpc_config_changes:
                ret = await hub.exec.boto3.client.eks.update_cluster_config(
                    ctx,
                    name=name,
                    resourcesVpcConfig=vpc_config_changes,
                    clientRequestToken=client_request_token,
                )
                result["result"] = ret["result"]
                if not result["result"]:
                    result["comment"] = ret["comment"]
                    return result
                await hub.tool.boto3.client.wait(
                    ctx,
                    "eks",
                    "cluster_active",
                    None,
                    name=name,
                    WaiterConfig={"MaxAttempts": update_attempts},
                )
                is_changed = True
            # update tags
            old_tags = (
                copy.deepcopy(before["ret"]["cluster"].get("tags"))
                if before["ret"]["cluster"].get("tags")
                else {}
            )

            update_tags_ret = await hub.exec.aws.eks.tag.update_eks_tags(
                ctx=ctx,
                resource_arn=arn,
                old_tags=old_tags,
                new_tags=tags,
            )
            if not update_tags_ret["result"]:
                result["comment"] = update_tags_ret["comment"]
                result["result"] = False
                return result
            if update_tags_ret["ret"]:
                is_changed = True
            if is_changed:
                result["comment"] = f"Updated '{name}'"
        else:
            try:
                ret = await hub.exec.boto3.client.eks.create_cluster(
                    ctx,
                    name=name,
                    version=version,
                    roleArn=role_arn,
                    resourcesVpcConfig=resources_vpc_config,
                    kubernetesNetworkConfig=kubernetes_network_config,
                    logging=logging,
                    encryptionConfig=encryption_config,
                    tags=tags,
                )
                result["result"] = ret["result"]
                if not result["result"]:
                    result["comment"] = ret["comment"]
                    return result
                await hub.tool.boto3.client.wait(
                    ctx,
                    "eks",
                    "cluster_active",
                    None,
                    name=name,
                    WaiterConfig={"MaxAttempts": create_attempts},
                )
                is_changed = True
                result["comment"] = f"Created '{name}'"
            except hub.tool.boto3.exception.ClientError as e:
                result["comment"] = f"{e.__class__.__name__}: {e}"
                result["result"] = False
    except Exception as e1:
        result["comment"] = f"{e1.__class__.__name__}: {e1}"
        result["result"] = False
        return result

    try:
        if is_changed:
            after = await hub.exec.boto3.client.eks.describe_cluster(ctx, name=name)
            result["new_state"] = after["ret"]["cluster"]
        else:
            result["new_state"] = copy.deepcopy(result["old_state"])
    except Exception as e:
        result["comment"] = str(e)
        result["result"] = False
    return result


async def absent(hub, ctx, name: str) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    Deletes the Amazon EKS cluster control plane. If you have active services in your cluster that are associated
    with a load balancer, you must delete those services before deleting the cluster so that the load balancers are
    deleted properly. Otherwise, you can have orphaned resources in your VPC that prevent you from being able to
    delete the VPC. For more information, see Deleting a Cluster in the Amazon EKS User Guide. If you have managed
    node groups or Fargate profiles attached to the cluster, you must delete them first. For more information, see
    DeleteNodegroup and DeleteFargateProfile.

    Args:
        name(Text): A name, ID to identify the resource.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            my_cluster:
              aws.eks.cluster.absent:
                - role_arn: arn:aws:iam::202236654445:role/ek1
                - version: '1.21'
    """

    result = dict(comment="", old_state=None, new_state=None, name=name, result=True)

    before = await hub.exec.boto3.client.eks.describe_cluster(ctx, name=name)

    if not before:
        result["comment"] = f"'{name}' already absent"
    elif ctx.get("test", False):
        result["comment"] = f"Would delete aws.eks.cluster {name}"
        return result
    else:
        result["old_state"] = before["ret"]["cluster"]
        try:
            ret = await hub.exec.boto3.client.eks.delete_cluster(
                ctx,
                name=name,
            )
            result["result"] = ret["result"]
            if not result["result"]:
                result["comment"] = ret["comment"]
                result["result"] = False
                return result
            await hub.tool.boto3.client.wait(
                ctx,
                "eks",
                "cluster_deleted",
                None,
                name=name,
            )
            result["comment"] = f"Deleted '{name}'"
        except hub.tool.boto3.exception.ClientError as e:
            result["comment"] = f"{e.__class__.__name__}: {e}"
    return result


async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
    r"""
    **Autogenerated function**

    Describe the resource in a way that can be recreated/managed with the corresponding "present" function


    Lists the Amazon EKS clusters in your Amazon Web Services account in the specified Region.


    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: bash

            $ idem describe aws.eks.cluster
    """

    result = {}
    ret = await hub.exec.boto3.client.eks.list_clusters(ctx)

    if not ret["result"]:
        hub.log.debug(f"Could not describe cluster {ret['comment']}")
        return {}
    describe_parameters = OrderedDict(
        {
            "roleArn": "role_arn",
            "status": "status",
            "version": "version",
            "endpoint": "endpoint",
        }
    )

    for name in ret["ret"]["clusters"]:
        describe_ret = await hub.exec.boto3.client.eks.describe_cluster(ctx, name=name)
        cluster = describe_ret["ret"]["cluster"]
        new_cluster = []
        for parameter_old_key, parameter_new_key in describe_parameters.items():
            if cluster.get(parameter_old_key) is not None:
                new_cluster.append({parameter_new_key: cluster.get(parameter_old_key)})
        if cluster.get("resourcesVpcConfig"):
            resources_vpc_config = cluster.get("resourcesVpcConfig").copy()
            resources_vpc_config.pop("vpcId", None)
            new_cluster.append({"resources_vpc_config": resources_vpc_config})
        if cluster.get("kubernetesNetworkConfig"):
            new_cluster.append(
                {
                    "kubernetes_network_config": cluster.get(
                        "kubernetesNetworkConfig"
                    ).copy()
                }
            )
        if cluster.get("logging"):
            new_cluster.append({"logging": cluster.get("logging").copy()})
        if cluster.get("encryptionConfig"):
            new_cluster.append(
                {"encryption_config": cluster.get("encryptionConfig").copy()}
            )
        if cluster.get("certificateAuthority"):
            new_cluster.append(
                {"certificate_authority": cluster.get("certificateAuthority").copy()}
            )
        if cluster.get("tags"):
            new_cluster.append({"tags": cluster.get("tags").copy()})
        result[name] = {"aws.eks.cluster.present": new_cluster}
    return result



from collections import OrderedDict
from typing import Any
from typing import Dict


async def convert_raw_key_to_present(
    hub, ctx, raw_resource: Dict[str, Any]
) -> Dict[str, Any]:
    describe_parameters = OrderedDict(
        {
            "KeyId": "resource_id",
            "KeyArn": "arn",
            "Arn": "arn",
            "KeyState": "key_state",
            "Description": "description",
            "KeyUsage": "key_usage",
            "KeySpec": "key_spec",
            "MultiRegion": "multi_region",
            "Policy": "policy",
            "Tags": "tags",
        }
    )

    translated_resource = {}
    for parameter_raw, parameter_present in describe_parameters.items():
        if parameter_raw in raw_resource:
            translated_resource[parameter_present] = raw_resource.get(parameter_raw)

    # Get key tags
    # For tag operations key_id is used and not resource_i
    key_tags = await hub.exec.boto3.client.kms.list_resource_tags(
        ctx, KeyId=translated_resource["resource_id"]
    )
    if key_tags and key_tags["result"] is True:
        translated_resource["tags"] = key_tags["ret"].get("Tags", [])

    # Get key policy
    key_policy = await hub.exec.boto3.client.kms.get_key_policy(
        ctx, KeyId=translated_resource["resource_id"], PolicyName="default"
    )
    if (
        key_policy
        and key_policy["result"] is True
        and key_policy["ret"].get("Policy", None)
    ):
        translated_resource[
            "policy"
        ] = hub.tool.aws.state_comparison_utils.standardise_json(
            key_policy["ret"].get("Policy")
        )

    # raw_resource may contains the details already, in this case,
    # skip describe_key call
    if translated_resource.get("key_state", None) is None:
        key_details = await hub.exec.boto3.client.kms.describe_key(
            ctx, KeyId=translated_resource["resource_id"]
        )
        if key_details and key_details["result"] is True:
            resource = key_details["ret"].get("KeyMetadata", {})
            for parameter_raw, parameter_present in describe_parameters.items():
                if parameter_raw in resource:
                    translated_resource[parameter_present] = resource.get(parameter_raw)

    return translated_resource


def convert_raw_key_alias_to_present(
    hub, raw_resource: Dict[str, Any]
) -> Dict[str, Any]:

    # Arn is not used for present but required for arg binding
    resource_parameters = OrderedDict(
        {
            "AliasArn": "arn",
            "TargetKeyId": "target_key_id",
        }
    )
    # AliasName is the unique identifier for KMS Alias, so it is set as resource_id
    translated_resource = {"resource_id": raw_resource.get("AliasName")}
    for parameter_raw, parameter_present in resource_parameters.items():
        if parameter_raw in raw_resource:
            translated_resource[parameter_present] = raw_resource.get(parameter_raw)

    return translated_resource



"""
Autogenerated using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__

hub.exec.boto3.client.iam.create_role
hub.exec.boto3.client.iam.delete_role
hub.exec.boto3.client.iam.get_role
hub.exec.boto3.client.iam.list_roles
hub.exec.boto3.client.iam.tag_role
hub.exec.boto3.client.iam.untag_role
hub.exec.boto3.client.iam.update_role
resource = hub.tool.boto3.resource.create(ctx, "iam", "Role", name)
hub.tool.boto3.resource.exec(resource, attach_policy, *args, **kwargs)
hub.tool.boto3.resource.exec(resource, delete, *args, **kwargs)
hub.tool.boto3.resource.exec(resource, detach_policy, *args, **kwargs)
"""
import copy
from typing import Any
from typing import Dict
from typing import List

__contracts__ = ["resource"]


async def present(
    hub,
    ctx,
    name: str,
    assume_role_policy_document: Dict or str,
    resource_id: str = None,
    description: str = None,
    max_session_duration: int = None,
    permissions_boundary: str = None,
    tags: List = None,
) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    Creates a new role for your Amazon Web Services account. For more information about roles, see IAM roles. For
    information about quotas for role names and the number of roles you can create, see IAM and STS quotas in the
    IAM User Guide.

    Args:
        name(Text): The name of the IAM role.
        assume_role_policy_document(Dict or Text): The trust relationship policy document that grants an entity
         permission to assume the role. This can be either a dictionary or a json string.
        resource_id(Text, Optional): The ID of the IAM role.
        description(Text, Optional): A description of the role. Defaults to None.
        max_session_duration(Integer, Optional): The maximum session duration (in seconds) that you want to set for the
         specified role. If you do not specify a value for this setting, the default maximum of one hour is applied.
          This setting can have a value from 1 hour to 12 hours.
        permissions_boundary(Text, Optional): The ARN of the policy that is used to set the permissions boundary for the role.
        tags(Text, Optional): A list of tags that you want to attach to the new role. Each tag consists of a key name and an associated value.

    Request Syntax:
        [iam-role-name]:
          aws.iam.role.present:
          - name: 'string'
          - resource_id: 'string'
          - assume_role_policy_document: 'dict or string'
          - description: 'string'
          - max_session_duration: 'integer'
          - permissions_boundary: 'string'
          - tags:
            - Key: 'string'
              Value: 'string'

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            AWSServiceRoleForEC2Spot:
              aws.iam.role.present:
                - assume_role_policy_document:
                  Statement:
                    - Action: sts:AssumeRole
                      Effect: Allow
                      Principal:
                        Service: spot.amazonaws.com
                  Version: '2012-10-17'
                - description: Default EC2 Spot Service Linked Role
                - max_session_duration: 3600
                - tags:
                  - Key: tag-key
                    Value: tag-value
    """
    result = dict(comment=(), old_state=None, new_state=None, name=name, result=True)
    resource_updated = False
    resource = hub.tool.boto3.resource.create(
        ctx, "iam", "Role", resource_id if resource_id else name
    )
    before = await hub.tool.boto3.resource.describe(resource)

    # Standardise on the json format of policy
    assume_role_policy_document = hub.tool.aws.state_comparison_utils.standardise_json(
        assume_role_policy_document
    )

    if before:
        result[
            "old_state"
        ] = hub.tool.aws.iam.conversion_utils.convert_raw_role_to_present(before)
        try:
            plan_state = copy.deepcopy(result["old_state"])
            # Update role
            update_ret = await hub.exec.aws.iam.role.update_role(
                ctx,
                old_state=result["old_state"],
                description=description,
                max_session_duration=max_session_duration,
            )
            result["comment"] = result["comment"] + update_ret["comment"]
            result["result"] = update_ret["result"]
            resource_updated = resource_updated or bool(update_ret["ret"])
            if update_ret["ret"] and ctx.get("test", False):
                if "max_session_duration" in update_ret["ret"]:
                    plan_state["max_session_duration"] = update_ret["ret"][
                        "max_session_duration"
                    ]
                if "description" in update_ret["ret"]:
                    plan_state["description"] = update_ret["ret"]["description"]

            # Update tags
            if (tags is not None) and (
                not hub.tool.aws.state_comparison_utils.are_lists_identical(
                    tags, result["old_state"].get("tags", None)
                )
            ):
                update_ret = await hub.exec.aws.iam.role.update_role_tags(
                    ctx,
                    role_name=before.get("RoleName"),
                    old_tags=before.get("Tags", []),
                    new_tags=tags,
                )
                result["result"] = result["result"] and update_ret["result"]
                result["comment"] = result["comment"] + update_ret["comment"]
                resource_updated = resource_updated or bool(update_ret["ret"])
                if ctx.get("test", False) and update_ret["ret"] is not None:
                    plan_state["tags"] = update_ret["ret"].get("tags")
            if not resource_updated:
                result["comment"] = result["comment"] + (
                    f"aws.iam.role '{name}' already exists",
                )
            # Update policy which is embedded within the role
            if not hub.tool.aws.state_comparison_utils.is_json_identical(
                result["old_state"]["assume_role_policy_document"],
                assume_role_policy_document,
            ):
                update_ret = await hub.exec.aws.iam.role.update_policy(
                    ctx,
                    role_name=result["old_state"].get("name", None),
                    policy=assume_role_policy_document,
                )
                result["result"] = result["result"] and update_ret["result"]
                result["comment"] = result["comment"] + update_ret["comment"]
                resource_updated = resource_updated or bool(update_ret["ret"])
                if ctx.get("test", False):
                    plan_state[
                        "assume_role_policy_document"
                    ] = assume_role_policy_document

        except hub.tool.boto3.exception.ClientError as e:
            result["comment"] = result["comment"] + (f"{e.__class__.__name__}: {e}",)
            result["result"] = False
    else:
        try:
            if ctx.get("test", False):
                result["new_state"] = hub.tool.aws.test_state_utils.generate_test_state(
                    enforced_state={},
                    desired_state={
                        "name": name,
                        "assume_role_policy_document": assume_role_policy_document,
                        "description": description,
                        "max_session_duration": max_session_duration,
                        "tags": tags,
                    },
                )
                result["comment"] = (f"Would create aws.iam.role '{name}'",)
                return result

            ret = await hub.exec.boto3.client.iam.create_role(
                ctx,
                RoleName=name,
                AssumeRolePolicyDocument=assume_role_policy_document,
                Description=description,
                MaxSessionDuration=max_session_duration,
                PermissionsBoundary=permissions_boundary,
                Tags=tags,
            )
            result["result"] = ret["result"]
            if not result["result"]:
                result["comment"] = result["comment"] + ret["comment"]
                return result
            result["comment"] = result["comment"] + (f"Created aws.iam.role '{name}'",)
        except hub.tool.boto3.exception.ClientError as e:
            result["comment"] = result["comment"] + (f"{e.__class__.__name__}: {e}",)
            result["result"] = False

    try:
        if ctx.get("test", False):
            result["new_state"] = plan_state
        elif (not before) or resource_updated:
            after = await hub.tool.boto3.resource.describe(resource)
            result[
                "new_state"
            ] = hub.tool.aws.iam.conversion_utils.convert_raw_role_to_present(after)
        else:
            result["new_state"] = copy.deepcopy(result["old_state"])
    except Exception as e:
        result["comment"] = result["comment"] + (str(e),)
        result["result"] = False
    return result


async def absent(hub, ctx, name: str, resource_id: str = None) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    Deletes the specified role. The role must not have any policies attached. For more information about roles, see
    Working with roles.  Make sure that you do not have any Amazon EC2 instances running with the role you are about
    to delete. Deleting a role or instance profile that is associated with a running instance will break any
    applications running on the instance.

    Args:
        name(Text): The name of the IAM role.
        resource_id(Text, Optional): unique identifier of the resource.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_absent:
              aws.iam.role.absent:
                - resource_id: value
    """

    result = dict(comment=(), old_state=None, new_state=None, name=name, result=True)
    resource = hub.tool.boto3.resource.create(
        ctx, "iam", "Role", resource_id if resource_id else name
    )
    before = await hub.tool.boto3.resource.describe(resource)

    if not before:
        result["comment"] = (f"aws.iam.role '{name}' already absent",)
    elif ctx.get("test", False):
        result[
            "old_state"
        ] = hub.tool.aws.iam.conversion_utils.convert_raw_role_to_present(before)
        result["comment"] = (f"Would delete aws.iam.role '{name}'",)
        return result
    else:
        try:
            result[
                "old_state"
            ] = hub.tool.aws.iam.conversion_utils.convert_raw_role_to_present(before)
            ret = await hub.exec.boto3.client.iam.delete_role(ctx, RoleName=name)
            result["result"] = ret["result"]
            if not result["result"]:
                result["comment"] = ret["comment"]
                result["result"] = False
                return result
            result["comment"] = (f"Deleted aws.iam.role '{name}'",)
        except hub.tool.boto3.exception.ClientError as e:
            result["comment"] = (f"{e.__class__.__name__}: {e}",)

    return result


async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
    r"""
    **Autogenerated function**

    Describe the resource in a way that can be recreated/managed with the corresponding "present" function


    Lists the IAM roles that have the specified path prefix. If there are none, the operation returns an empty list.
    For more information about roles, see Working with roles.  IAM resource-listing operations return a subset of
    the available attributes for the resource. For example, this operation does not return tags, even though they
    are an attribute of the returned object. To view all of the information for a role, see GetRole.  You can
    paginate the results using the MaxItems and Marker parameters.


    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: bash

            $ idem describe aws.iam.role
    """

    result = {}
    ret = await hub.exec.boto3.client.iam.list_roles(ctx)

    if not ret["result"]:
        hub.log.debug(f"Could not describe role {ret['comment']}")
        return {}

    for role in ret["ret"]["Roles"]:
        # This is required to get tags for each role
        boto2_resource = hub.tool.boto3.resource.create(
            ctx, "iam", "Role", role.get("RoleName")
        )
        resource = await hub.tool.boto3.resource.describe(boto2_resource)
        translated_resource = (
            hub.tool.aws.iam.conversion_utils.convert_raw_role_to_present(resource)
        )

        result[translated_resource["resource_id"]] = {
            "aws.iam.role.present": [
                {parameter_key: parameter_value}
                for parameter_key, parameter_value in translated_resource.items()
            ]
        }

    return result
